/* Copyright 2007, 2008 DJ Delorie <dj@delorie.com>
   Distributed under the terms of the GNU General Public License
   version 3, see COPYING-3.0 for details.
*/
#include "pins.h"
#include "funcs.h"
#include "globals.h"
#include "TCPIP Stack/TCPIP.h"

static volatile int adc_complete = 0;
static int ad0, ad5, ad11;
static int button_tick = 0;

char adc_debug = 0;
byte button = 0;
byte button_autorepeat = 0;
byte button_event = 0;

static TICK button_timeout = 0;
#define BUTTON_TIMEOUT_TIME (15 * TICK_SECOND)

int volume = 0;
byte photocell = 15;
static byte old_photocell = 0xfe;
static int old_ad11 = -1;
#define PHOTO_HYST 32

static byte debounce = 0;
#define DEBOUNCE_COUNT 10

#define AREPEAT1 500
static int next_repeat;

static int cnt[2];
static int last_button = -1;

static int
dejitter (int which, int val, int range)
{
  if (val > cnt[which] + range)
    cnt[which] = val - range;
  if (val < cnt[which] - range)
    cnt[which] = val + range;
  return cnt[which];
}

/* CODE SAMPLE: adc_task.exc */

void
adc_task ()
{
  if (!adc_complete)
    return;

  if (AD1CON2bits.BUFS)
    {
      ad0 = ADC1BUF0;
      ad5 = ADC1BUF1;
      ad11 = ADC1BUF2;
    }
  else
    {
      ad0 = ADC1BUF8;
      ad5 = ADC1BUF9;
      ad11 = ADC1BUFA;
    }

  if (adc_debug)
    {
      local_console ++;
      gotoxy (14, 0);
      cprintf("B:%w", ad5);
      gotoxy (14, 1);
      cprintf("V:%w", ad0);
      gotoxy (14, 2);
      cprintf("P:%w", ad11);
      local_console --;
    }

  // AD5 is the button input
  ad5 >>= 7;
  if (ad5 == last_button)
    {
      if (debounce < DEBOUNCE_COUNT)
	debounce++;
      else
	{
	  int new_button = (ad5 + 1) & 7;
	  // Sometimes the rising edge is slow enough to trigger a higher
	  // button number.  We only support changes to/from "no button".
	  if (button != new_button && !(button && new_button))
	    {
	      button = new_button;
	      button_tick = iTick () + AREPEAT1;
	      next_repeat = AREPEAT1;
	      if (button)
		{
		  button_event = 1;
		  button_timeout = 0;
		}
	      else
		{
		  button_timeout = sTick ();
		  if (button_timeout == 0)
		    button_timeout = 1;
		}
	    }
	  else if (!button && button_timeout
		   && (long) TickGet () - button_timeout >
		   BUTTON_TIMEOUT_TIME)
	    {
	      // A button event of button 0 means "timeout"
	      button_event = 1;
	      button_timeout = 0;
	    }
	}
    }
  else
    debounce = 0;

  last_button = ad5;
  if (button_autorepeat & (1 << button))
    {
      if (iTick () - button_tick > 0)
	{
	  if (next_repeat > 100)
	    next_repeat -= (next_repeat >> 3);
	  button_tick = iTick () + next_repeat;
	  button_event = 1;
	}
    }

  // AD0 is the volume sensor
  ad0 = dejitter (0, ad0, 8);

  ad0 = (ad0 - 134) * 86 / 16;
  if (ad0 < 0)
    ad0 = 0;
  if (ad0 > 0x3ff)
    ad0 = 0x3ff;
  volume = ad0;
  todo.check_volume = 1;

  // AD11 is the photocell
  if (ad11 < (old_photocell ? 4 : 16))
    photocell = 0;
  else
    {
      ad11 = (ad11 + old_ad11) / 2;

      photocell = ((ad11 + 32) >> 6) + (ad11 >> 9);
      if (photocell > 15)
	photocell = 15;
      if (photocell < 1)
	photocell = 1;
    }
  if (photocell != old_photocell)
    {
      old_ad11 = ((int) photocell << 6) + 0x20;
      old_photocell = photocell;
      oled_set_intensity ();
      todo.rtc_need_redisplay = 1;
    }

  adc_complete = 0;
}

void __attribute__ ((interrupt)) _ADC1Interrupt (void)
{
  IFS0bits.AD1IF = 0;
  adc_complete = 1;
}

/* END CODE SAMPLE */
