/* Copyright 2007, 2008 DJ Delorie <dj@delorie.com>
   Distributed under the terms of the GNU General Public License
   version 3, see COPYING-3.0 for details.
*/
#include "pins.h"

_CONFIG2(IESO_OFF & FNOSC_FRCPLL & FCKSM_CSDCMD & OSCIOFNC_ON & IOL1WAY_OFF & I2C1SEL_PRI & POSCMOD_NONE)
_CONFIG1(JTAGEN_OFF & GCP_OFF & GWRP_OFF & BKBUG_OFF & COE_OFF & ICS_PGx2 & FWDTEN_OFF & WINDIS_OFF & FWPSA_PR128 & WDTPS_PS32768);

// This file configures all the pins for their various functions, and
// configures the hardware devices that will talk to those pins.  Pins
// below are listed by pin number, skipping power pins.

     void
     configure_pic_io ()
{
  TRISA = TRISB = ~0;
  AD1PCFG = 0xffff;

  // 1: MCLR

  // 2: RA0 is volume/mute, which is either an AD9 input or a GPIO
  // output pull-down.

  _TRISA0 = 1;			// GPIO input
  _LATA0 = 0;			// pull down mutes the volume

  // 3: RA1 is oled_dc, GPIO output

  _TRISA1 = 0;
  _LATA1 = 0;			// command mode

/* CODE SAMPLE: config.exc */

  // 4: RB0 is oled_cs, GPIO ouput

  _TRISB0 = 0;
  _LATB0 = 1;			// active low, cs inactive

  // 5: RP1 is SPI2-CK (output)

  RPOR0bits.RP1R = 11;		// SPI2 clock output
  SPI2CON1 = 0x013e;		// ck idle low, sample rising, change falling, 1:4
  SPI2CON2 = 0x0001;		// no framing, enhanced mode
  SPI2STAT = 0x8004;		// enable, interrupt on rx ready

  // 6: RP2 is SPI2-D0 (output)

  RPOR1bits.RP2R = 10;		// SPI2 data out

  // 7: AN5 is the key press input

  _TRISB3 = 1;
  AD1PCFG = 0xF7DE;		// AN0, AN5, and AN11 are analog
  AD1CON1 = 0x00E4;		// off, integer, auto-convert
  AD1CON2 = 0x040a;		// Vref: Vdd/Vss,scan, int each 3, split
  AD1CON3 = 0x9Fff;		// RCclk, 31 Tad, 128 Tcy
  AD1CHS = 0x0000;
  AD1CSSL = 0x0821;		// all channels selected for scan
  IEC0bits.AD1IE = 1;
  IPC3bits.AD1IP = 3;
  AD1CON1bits.ADON = 1;		// turn on the ADC
  AD1CON1bits.SAMP = 1;		// start sampling

/* END CODE SAMPLE */

  // 9: RA2 is reset, GPIO output

  _TRISA2 = 0;
  _LATA2 = 0;			// Active low, resetting

  // 10: RA3 is v12en, GPIO output

  _TRISA3 = 0;
  _LATA1 = 0;			// 12v is off

  // 11: SOSCI is 32kHz input
  // 12: SOSCO is 32kHz output

  // 14: SDA1 for I2C (RB5)
  // 15: SCL1 for I2C (RB6)

  _TRISB5 = _TRISB6 = 1;	// float
  _LATB5 = _LATB6 = 0;		// or pull down
  //I2C1BRG = 79;               // aim for 400 KHz
  //I2C1CON = 0x9200;           // enable I2C master

  // Timer for soft-i2c
  PR1 = 0xffff;
  TMR1 = 0;
  IEC0bits.T2IE = 0;
  IFS0bits.T2IF = 0;
  IPC1bits.T2IP = 0;
  T2CON = 0x0;
  T2CONbits.TON = 1;

  // 16: RB7 is ether_cs, GPIO output

  _TRISB7 = 0;
  _LATB7 = 1;			// active low, cs inactive

  // 17: RP8 is SPI1-C (output)

  RPOR4bits.RP8R = 8;		// SPI1 clock out
  SPI1CON1 = 0x013b;		// ck idle low, sample rising, change falling, 1:2
  SPI1CON2 = 0x0001;		// no framing, enhanced mode
  SPI1STAT = 0x8004;		// enable, interrupt on rx ready

  // 18: RP9 is SPI1-DO (output)

  RPOR4bits.RP9R = 7;		// SPI1 data out

  // 21: RP10 is U1TX for the console (output)
  // and ICE

#if 1
  _TRISB10 = 0;
  _LATB10 = 0;
#else
  RPINR18bits.U1RXR = 10;
  RPOR5bits.RP10R = 3;		// uart1 TX
#endif
  //U1BRG  = 0;			// 1Mbaud
  U1BRG = 34;			// 115,200
  U1MODE = 0x8008;		// 8N1, high speed, enable 
  U1STA = 0x0440;		// enable TX
  _U1RXIF = 0;
  _TRISB10 = 0;			// output

  // 22: RB11 is ICE

  _TRISB11 = 0;

  // 23: RP12 is SPI1-DI (input)

  RPINR20bits.SDI1R = 12;
  _TRISB12 = 1;

  // 24: AN11 is the photocell input (RB13)

  _TRISB13 = 1;			// GPIO input

  // 25: RB14 is mp3rdy, GPIO input

  _TRISB14 = 1;

  // 26: RB15 is mp3_cs, GPIO output

  _TRISB15 = 0;
  _LATB15 = 0;			// active high, cs inactive

  // Don't divide the clock; run fast
  CLKDIVbits.RCDIV = 0;
  CLKDIVbits.DOZE = 0;
  CLKDIVbits.ROI = 1;
}
