/* Copyright 2007, 2008 DJ Delorie <dj@delorie.com>
   Distributed under the terms of the GNU General Public License
   version 3, see COPYING-3.0 for details.
*/
#include <stdarg.h>

#include "pins.h"
#include "funcs.h"
#ifndef SIM
#include "TCPIP Stack/TCPIP.h"
#endif /* not SIM */

/*#undef SYSLOG*/
#define SYSLOG

static char log_buf[80];
static int logp = 0;
#ifdef SYSLOG
#ifndef SIM
static UDP_SOCKET LOGSocket = INVALID_UDP_SOCKET;
static NODE_INFO envy = {
  { { 0x00, 0x18, 0xF3, 0x9D, 0x84, 0x23 } },
  { .v = { 207, 22, 48, 171 } }
};
#endif

static int cloop = 0;
#endif

void
cput_logc (char c)
{
  console_send_cr(c);
#ifndef SIM
#ifdef SYSLOG
  if (cloop)
    return;
  log_buf[logp++] = c;
  if (c != '\n' && logp < 80)
    return;

  cloop = 1;

  LOGSocket = UDPOpen (514, &envy, 514);
  if (LOGSocket != INVALID_UDP_SOCKET)
    {
      if (UDPIsPutReady (LOGSocket) >= logp+4)
	{
	  UDPPutArray ((BYTE *)"<133>", 5);
	  UDPPutArray ((BYTE *)log_buf, logp);
	  UDPFlush ();
	}
      UDPClose(LOGSocket);
    }
  logp = 0;
  cloop = 0;
#endif
#endif
}

char local_console = 0;

#define cput_c(x) cputc_switch(x)
static void
cputc_switch (char c)
{
  if (local_console)
    (cput_c) (c);
  else
    cput_logc (c);
}

void
cput_crlf ()
{
  cput_c ('\n');
}

void
cput_s (const char *s)
{
  while (*s)
    {
      cput_c (*s);
      s++;
    }
}

static char hex[] = "0123456789abcdef";

void
cput_nibble (int n)
{
  cput_c (hex[n & 0x0f]);
}

void
cput_hex_byte (int n)
{
  cput_nibble (n >> 4);
  cput_nibble (n);
}

void
cput_hex_word (int n)
{
  cput_hex_byte (n >> 8);
  cput_hex_byte (n);
}

void
cput_hex_block (byte * block, int n)
{
  int i = 0;
  while (n)
    {
      cput_hex_byte (*block++);
      if (--n == 0)
	break;
      i++;
      if ((i & 7) == 0)
	cput_c (' ');
      else
	cput_c (':');
    }
}

void
cput_nibble_block (byte * block, int n)
{
  int i = 0;
  while (n)
    {
      cput_nibble (*block);
      if (--n == 0)
	break;
      i++;
      if ((i & 7) == 0)
	cput_c (' ');
    }
}

void
cput_number (int n)
{
  char buf[7];
  int i = 0;
  if (n < 0)
    {
      cput_c ('-');
      n = -n;
    }
  while (n > 9)
    {
      buf[i++] = (n % 10) + '0';
      n /= 10;
    }
  buf[i++] = (n % 10) + '0';
  while (i > 0)
    cput_c (buf[--i]);
}

void
cprintf (const char *fmt, ...)
{
  va_list v;
  int i;
  char *s;
  unsigned char *us;

  va_start (v, fmt);

  while (*fmt)
    {
      if (*fmt != '%')
	cput_c (*fmt);
      else
	switch (*++fmt)
	  {
	  case '%':
	    cput_c ('%');
	    break;
	  case 'c':
	    i = va_arg (v, int);
	    cput_c (i);
	    break;
	  case 'd':
	    i = va_arg (v, int);
	    cput_number (i);
	    break;
	  case 'b':
	    i = va_arg (v, int);
	    cput_hex_byte (i);
	    break;
	  case 'w':
	    i = va_arg (v, int);
	    cput_hex_word (i);
	    break;
	  case 'x':
	    us = va_arg (v, byte *);
	    i = va_arg (v, int);
	    cput_hex_block (us, i);
	    break;
	  case 'n':
	    us = va_arg (v, byte *);
	    i = va_arg (v, int);
	    cput_nibble_block (us, i);
	    break;
	  case 's':
	    s = va_arg (v, char *);
	    cput_s (s);
	    break;
	  case 'P':
	    us = va_arg (v, unsigned char *);
	    cput_number (us[0]);
	    cput_c ('.');
	    cput_number (us[1]);
	    cput_c ('.');
	    cput_number (us[2]);
	    cput_c ('.');
	    cput_number (us[3]);
	    break;
	  }
      fmt++;
    }
}
