/* Copyright 2007, 2008 DJ Delorie <dj@delorie.com>
   Distributed under the terms of the GNU General Public License
   version 3, see COPYING-3.0 for details.
*/
typedef unsigned char byte;
typedef unsigned short word;
typedef unsigned int uint;

/* main.c */
void waitus (int i);
void waitms (int i);

void console_send (char c);
void console_send_cr (char c);
void console_nibble (int i);
void console_hex (int i);
void console_wait (void);

/* config.c */
void configure_pic_io (void);

/* i2c.c */
int i2c_rd (int addr, void *buf, uint len);
int i2c_wr (int addr, void *buf, uint len);
int i2c_wra (int addr, uint ptr, void *buf, uint len);
int i2c_srd (int addr);
void i2c_end (void);

/* oled.c */
void cls (void);
void gotoxy (int x, int y);
void put_c_xy (int c, int x, int y);
void cput_c (char c);
void cput_cs (const char *c);
void configure_oled (void);
void oled_demo (void);
void oled_fill (int x1, int y1, int x2, int y2, int val);
void oled_image (uint iaddr, int x, int y);
void oled_set_intensity (void);

/* console.c */
void cput_crlf (void);
void cput_s (const char *s);
void cput_nibble (int n);
void cput_hex_byte (int n);
void cput_hex_word (int n);
void cput_hex_block (byte * block, int n);
void cput_nibble_block (byte * block, int n);
void cput_numbmer (int n);
void cprintf (const char *fmt, ...);

/* spi.c */
void spi_oled_data (void);
void spi_oled_command (void);
void spi_mp3 (void);
void spi_net (void);
void spi_putc (char byte);
void ospi_putc (char byte);
void spi_puts (const char *byte, int len);
void ospi_puts (const char *byte, int len);
void spi_done (void);

/* mp3.c */
void mp3_init (void);
void mp3_task (void);
void mp3_process_buffer (void);
void mp3_set_mute (int muted);
void mp3_play_beep (void);
void mp3_play_url (void);
void mp3_stop (void);
void mp3_zen_timer_start (int zen_setting);
void mp3_zen_timer_tick (void);
void mp3_zen_volume (int);

/* tcpip.c */
void InitAppConfig (void);

/* rtc.c */
typedef enum
{
  RTC_NONE, RTC_MAIN, RTC_HALF, RTC_MINI, RTC_VOL
} RTC_DISP;
void rtc_init (void);
void rtc_task (void);
/* X,Y is the top center of the display */
void rtc_disp (RTC_DISP disp, int x, int y);
RTC_DISP rtc_current_mode (void);
/* hour and minute are BCD */
void rtc_disp_hm (RTC_DISP disp, int x, int y, byte hour, byte minute);
void rtc_disp_hm24 (RTC_DISP disp, int x, int y, byte hour, byte minute);
void rtc_pause (int paused);
int hour_24_to_12 (int hour, byte * pm);
void rtc_print_date (void);

/* adc.c */
void adc_task (void);

/* eeprom.c */
extern void eeprom_write (uint address, void *ptr, int len);
extern void eeprom_read (uint address, void *ptr, int len);
extern int eeprom_readc (uint address);
extern void eeprom_init (void);
extern void eeprom_task (void);

extern uint get_toc (int idx, uint * sizep);

/* alarm.c */
extern void alarm_task (void);
extern void alarm_check (void);
// index of web destination
extern void trigger_alarm_i (int index);
// address of web destination
extern void trigger_alarm (uint address, int zen);
extern void alarm_off (void);
extern void alarm_reconnect (void);
extern void alarm_snooze (void);
extern void sleep_on (void);
extern void sleep_off (void);

/* gui.c */
extern void gui (int button);
extern void update_sleep_time_display (void);

#define bcd(x) ((x)%10 + 16*((int)(x)/10))
#define unbcd(x) (((x)&15) + 10*((int)(x)>>4))

#define MIN(a,b) ((a) < (b) ? (a) : (b))
#define MAX(a,b) ((a) > (b) ? (a) : (b))
#define CLIP(a,b,c) MIN (MAX (a, b), c)

#if 0
#define Q(x) console_send(x), console_send(8)
#define QQ(x) console_send(x)
#else
#define Q(x)
#define QQ(x)
#endif

#define sTick() ((long)TickGet())
#define iTick() ((int)TickGet())
