/* Copyright 2007, 2008 DJ Delorie <dj@delorie.com>
   Distributed under the terms of the GNU General Public License
   version 3, see COPYING-3.0 for details.
*/
#include "pins.h"
#include "funcs.h"
#include "globals.h"

#ifndef SIM
#include "TCPIP Stack/TCPIP.h"
extern APP_CONFIG AppConfig;
#endif

static int alarm_idx, alarm_scroll;

static byte in_sleep_gui = 0;

// Max sleep time is five hours.  This has to fit into a signed in
// without being within 15 minutes of overflowing.
#define MAX_SLEEP (5*60*60)

#define RT_ARROW 29
#define LF_ARROW 27
#define BRT_ARROW 8
#define BLF_ARROW 6

typedef enum
{
  ASF_hour,
  ASF_minute,
  ASF_sun,
  ASF_mon,
  ASF_tue,
  ASF_wed,
  ASF_thu,
  ASF_fri,
  ASF_sat,
  ASF_month,
  ASF_mday,
  ASF_zen,
  ASF_webdest
} Alarm_Set_Field;

static byte days_per_month[] = { 0,	/* months start with 1 */
  31, 29, 31, 30, 31, 30, 31, 31, 30,
  31, 30, 31, 0, 0, 0,		/* these are for decimal months */
  31, 30, 31, 0, 0, 0		/* these are for BCD months */
};
static char month_names[] = "JanFebMarAprMayJunJulAugSepOctNovDec";

static Alarm_Set_Field alarm_set_field;

#define GOTO(x) state = &&x; cprintf("%d: goto %s from %d\n", button, #x, __LINE__); return;
#define WAIT() state = &&temp##__LINE__; cprintf("%d: wait at %d\n", button, __LINE__); return; temp##__LINE__:;
#define RESET() reset_display(); GOTO(idle_s)
#define IMAGE(x) oled_image (get_toc(x, 0), 0, 0);

static void
reset_display ()
{
  in_sleep_gui = 0;
  cls ();
  button_autorepeat = 0;
  rtc_pause (0);
  rtc_disp (RTC_MAIN, 64, 0);
}

#define icon(b,i) icon1(b,TOC_image_##i)
static void
icon1 (int button, int image)
{
  uint addr = get_toc (image, 0);
  switch (button)
    {
    case 1:
      oled_image (addr, 0, 0);
      break;
    case 2:
      oled_image (addr, 0, 22);
      break;
    case 3:
      oled_image (addr, 0, 44);
      break;
    case 4:
      oled_image (addr, 112, 0);
      break;
    case 5:
      oled_image (addr, 112, 22);
      break;
    case 6:
      oled_image (addr, 112, 44);
      break;
    }
}

static void
asf_put (Alarm_Set_Field asf)
{
  uint addr;
  switch (asf)
    {
    case ASF_hour:
      addr = get_toc (TOC_image_rarrow16, 0);
      oled_image (addr, 16, 9);
      break;
    case ASF_minute:
      addr = get_toc (TOC_image_larrow16, 0);
      oled_image (addr, 100, 9);
      break;
    case ASF_sun:
      put_c_xy (RT_ARROW, 3, 4);
      break;
    case ASF_mon:
      put_c_xy (RT_ARROW, 5, 4);
      break;
    case ASF_tue:
      put_c_xy (RT_ARROW, 7, 4);
      break;
    case ASF_wed:
      put_c_xy (RT_ARROW, 9, 4);
      break;
    case ASF_thu:
      put_c_xy (RT_ARROW, 11, 4);
      break;
    case ASF_fri:
      put_c_xy (RT_ARROW, 13, 4);
      break;
    case ASF_sat:
      put_c_xy (RT_ARROW, 15, 4);
      break;
    case ASF_month:
      put_c_xy (RT_ARROW, 3, 5);
      break;
    case ASF_mday:
      put_c_xy (RT_ARROW, 8, 5);
      break;
    case ASF_zen:
      put_c_xy (RT_ARROW, 12, 5);
      break;
    case ASF_webdest:
      put_c_xy (RT_ARROW, 3, 6);
      break;
    }
}

static void
asf_clear (Alarm_Set_Field asf)
{
  switch (asf)
    {
    case ASF_hour:
      oled_fill (16, 9, 28, 19, 0);
      break;
    case ASF_minute:
      oled_fill (100, 9, 110, 19, 0);
      break;
    case ASF_sun:
      put_c_xy (' ', 3, 4);
      break;
    case ASF_mon:
      put_c_xy (' ', 5, 4);
      break;
    case ASF_tue:
      put_c_xy (' ', 7, 4);
      break;
    case ASF_wed:
      put_c_xy (' ', 9, 4);
      break;
    case ASF_thu:
      put_c_xy (' ', 11, 4);
      break;
    case ASF_fri:
      put_c_xy (' ', 13, 4);
      break;
    case ASF_sat:
      put_c_xy (' ', 15, 4);
      break;
    case ASF_month:
      put_c_xy (' ', 3, 5);
      break;
    case ASF_mday:
      put_c_xy (' ', 8, 5);
      break;
    case ASF_zen:
      put_c_xy (' ', 12, 5);
      break;
    case ASF_webdest:
      put_c_xy (' ', 3, 6);
      break;
    }
}

static void
list_alarms ()
{
  static char weekdays[] = "SMTWTFS";
  uint addr, size, count, i, c;
  byte pm;
  int h12;
  AlarmStruct alarm;

  addr = get_toc (TOC_alarm, &size);
  count = size / sizeof (alarm);

  if (alarm_idx < 0)
    alarm_idx = 0;
  if (alarm_idx >= count)
    alarm_idx = count - 1;
  if (alarm_idx < alarm_scroll)
    alarm_scroll = alarm_idx;
  if (alarm_idx >= alarm_scroll + 8)
    alarm_scroll = alarm_idx - sizeof (alarm);
  if (alarm_scroll < 0)
    alarm_scroll = 0;

  for (i = 0; i < 8; i++)
    {
      int idx = i + alarm_scroll;

      gotoxy (3, i);

      if (idx >= count)
	{
	  cput_cs ("               ");
	  continue;
	}

      eeprom_read (addr + idx * sizeof (alarm), &alarm, sizeof (alarm));

      cput_c (1);
      cput_c (alarm.dow & 0x80 ? 1 : 2);
      cput_c ((idx == alarm_idx) ? RT_ARROW : ' ');
      if (alarm.dow == 0 && alarm.month == 0)
	{
	  cput_cs ("<empty>       ");
	  continue;
	}
      h12 = hour_24_to_12 (alarm.hour, &pm);
      c = h12 >> 4;
      cput_c (c ? c + '0' : ' ');
      cput_c ((h12 & 0x0f) + '0');
      cput_c (':');
      cput_c ((alarm.minute >> 4) + '0');
      cput_c ((alarm.minute & 0x0f) + '0');
      pm = pm ? 'P' : 'A';
      if (alarm.hour == 0 && alarm.minute == 0)
	pm = 'M';
      if (alarm.hour == 0x12 && alarm.minute == 0)
	pm = 'N';
      cput_c (pm);

      cput_c (' ');
      if (alarm.month)
	{
	  char *mp = month_names + unbcd (alarm.month) * 3 - 3;
	  cput_c (mp[0]);
	  cput_c (mp[1]);
	  cput_c (mp[2]);
	  cput_c (' ');
	  if (alarm.day > 9)
	    cput_c ((alarm.day >> 4) + '0');
	  else
	    cput_c (' ');
	  cput_c ((alarm.day & 15) + '0');
	  cput_c (' ');
	}
      else
	{
	  for (c = 0; c < 7; c++)
	    {
	      if (alarm.dow & (0x40 >> c))
		cput_c (weekdays[c]);
	      else
		cput_c ('-');
	    }
	}
    }
}

void
update_sleep_time_display (void)
{
  int m;
  if (!in_sleep_gui)
    return;
  m = (sleep_time + 15) / 60;
  rtc_disp_hm24 (RTC_HALF, 56, 16, bcd (m / 60), bcd (m % 60));
}

static void
update_sleep_volume_bars (void)
{
  uint addr = get_toc (TOC_config, 0);
  byte volume = eeprom_readc (addr + ConfigOffset (sleep_volume));
  volume /= 4;
  volume = 63 - volume;
  if (volume)
    oled_fill (97, 0, 107, volume - 1, 0x66);
  oled_fill (97, volume, 107, 63, 0xff);
}

// The button layout is:
//           7
//  1  +-----------+  4
//  2  |           |  5
//  3  +-----------+  6

// Yes, this is a giant state machine strife with goto's.
// Deal with it ;-)

void
gui (int button)
{
  static void *state = &&idle_s;
  static AlarmStruct alarm;
  uint addr, waddr;
  int i, delta, volume;
  char timezone;
  byte b;

  //cprintf("\033[32mgui: button %d\033[0m\n", button);
  goto *state;

  // Default idle state - display main clock, wait for input.
idle_s:
  switch (button)
    {
    case 1:
      alarm_idx = alarm_scroll = 0;
      goto pick_alarm_i;
      return;
    case 2:
      mp3_set_mute (!status.muted);
      return;
    case 3:
      trigger_alarm_i (1);
      return;
    case 4:
      alarm_off ();
      return;
    case 5:
      goto sleep_timer_i;
      return;
    case 6:
      trigger_alarm_i (2);
      return;
    default:
      return;
    }

sleep_timer_i:
  cls ();
  button_autorepeat = B2 | B3 | B5 | B6;
  icon (1, iok);
  icon (2, iup);
  icon (3, idown);
  icon (4, ionoff);
  icon (5, ilouder);
  icon (6, iquieter);
  in_sleep_gui = 1;
  update_sleep_time_display ();
  update_sleep_volume_bars ();
  GOTO (sleep_timer_s);
sleep_timer_s:
  switch (button)
    {
    case 2:
      // 15 minutes more
      if (sleep_time < MAX_SLEEP)
	sleep_time += 15 * 60;
      if (sleep_time <= 15 * 60)
	sleep_on ();
      update_sleep_time_display ();
      return;
    case 3:
      // 15 minutes less
      sleep_time -= 15 * 60;
      if (sleep_time <= 0)
	{
	  sleep_time = 0;
	  sleep_off ();
	}
      update_sleep_time_display ();
      return;

    case 4:
      sleep_time = 0;
      sleep_off ();
      RESET ();

    case 5:
      // volume up
      delta = 4;
      goto sleep_volume_change;
    case 6:
      // volume down
      delta = -4;
    sleep_volume_change:
      addr = get_toc (TOC_config, 0);
      volume = eeprom_readc (addr + ConfigOffset (sleep_volume));
      volume += delta;
      if (volume < 0)
	volume = 0;
      if (volume > 255)
	volume = 255;
      b = volume;
      eeprom_write (addr + ConfigOffset (sleep_volume), &b, 1);
      update_sleep_volume_bars ();
      mp3_zen_volume (volume);
      return;

    case 1:
    default:
      RESET ();
    }

  // Pick an alarm to change
pick_alarm_i:
  cls ();
  button_autorepeat = B2 | B3;
  icon (1, iok);
  icon (2, iup);
  icon (3, idown);
  icon (4, icancel);
  icon (5, ionoff);
  icon (6, iset);
  rtc_pause (1);
pick_alarm_i2:
  list_alarms ();
  GOTO (pick_alarm_s);
pick_alarm_s:
  switch (button)
    {
    case 1:
      goto debug_time_i;
    case 2:
      // prev_alarm
      alarm_idx--;
      goto pick_alarm_i2;
    case 3:
      // next_alarm
      alarm_idx++;
      goto pick_alarm_i2;
    case 4:
    case 0:
      RESET ();
    case 5:
      {
	// on/off
	uint addr = get_toc (TOC_alarm, 0);
	byte mask =
	  eeprom_readc (addr + alarm_idx * sizeof (AlarmStruct) + 2);
	mask ^= 0x80;
	eeprom_write (addr + alarm_idx * sizeof (AlarmStruct) + 2, &mask, 1);
	goto pick_alarm_i2;
      }
    case 6:
      goto set_alarm_i;
    }

  // Change an alarm
set_alarm_i:
  cls ();
  button_autorepeat = B2 | B3 | B5 | B6;
  icon (1, iok);
  icon (2, iup);
  icon (3, idown);
  icon (4, icancel);
  icon (5, iright);
  icon (6, ileft);
  alarm_set_field = ASF_hour;
  asf_put (ASF_hour);
set_alarm_i2:
  addr = get_toc (TOC_alarm, 0);
  eeprom_read (addr + alarm_idx * sizeof (alarm), &alarm, sizeof (alarm));
  cprintf ("alarm %d %w %b:%b %b %b/%b %w\n",
	   alarm_idx, addr + alarm_idx * sizeof (alarm),
	   alarm.hour, alarm.minute, alarm.dow, alarm.month, alarm.day,
	   alarm.web);
  rtc_disp_hm (RTC_HALF, 64, 0, alarm.hour, alarm.minute);
  cput_c (1);
  for (i = 0; i < 7; i++)
    if (alarm.dow & (0x40 >> i))
      put_c_xy ("SMTWTFS"[i] | 0x80, 4 + i * 2, 4);
    else
      put_c_xy ('-', 4 + i * 2, 4);
  if (alarm.month)
    {
      int mi = unbcd (alarm.month);
      char *mp = month_names + mi * 3 - 3;
      put_c_xy (mp[0] | 0x80, 4, 5);
      put_c_xy (mp[1] | 0x80, 5, 5);
      put_c_xy (mp[2] | 0x80, 6, 5);
      if (alarm.day > 9)
	put_c_xy ((alarm.day >> 4) + '0' + 0x80, 9, 5);
      else
	put_c_xy (' ', 9, 5);
      put_c_xy ((alarm.day & 15) + '0' + 0x80, 10, 5);
    }
  else
    {
      put_c_xy ('-', 4, 5);
      put_c_xy ('-', 5, 5);
      put_c_xy ('-', 6, 5);
      put_c_xy ('-', 9, 5);
      put_c_xy ('-', 10, 5);
    }

  put_c_xy ((alarm.zen / 60) + '0' + 0x80, 13, 5);
  put_c_xy (':' + 0x80, 14, 5);
  put_c_xy (((alarm.zen % 60) / 10) + '0' + 0x80, 15, 5);
  put_c_xy ((alarm.zen % 10) + '0' + 0x80, 16, 5);
  put_c_xy ('z' + 0x80, 17, 5);

  waddr = alarm.web;
  cprintf ("waddr %w\n", waddr);
  if (waddr)
    {
      WebDestination web;
      int idx = waddr - get_toc (TOC_web, 0);
      idx /= sizeof (WebDestination);
      eeprom_read (waddr, &web, sizeof (web));
      cprintf ("name %w\n", web.name_addr);
      if (web.name_addr)
	{
	  word slen, i;
	  char buf[15];
	  buf[14] = 0;
	  eeprom_read (web.name_addr, &slen, 2);
	  eeprom_read (web.name_addr + 2, buf, slen < 14 ? slen : 14);
	  cprintf ("name %d %s\n", slen, buf);
	  for (i = 0; i < slen && i < 14; i++)
	    put_c_xy (buf[i] + 0x80, 4 + i, 6);
	  for (; i < 14; i++)
	    put_c_xy (' ' + 0x80, 4 + i, 6);
	}
      else
	{
	  put_c_xy ('#' | 0x80, 4, 6);
	  put_c_xy ((idx / 10) + '0' + 0x80, 5, 6);
	  put_c_xy ((idx % 10) + '0' + 0x80, 6, 6);
	  gotoxy (7, 6);
	  cput_cs ("           ");
	}
    }
  else
    {
      gotoxy (4, 6);
      cput_cs ("<beep>        ");
    }

  GOTO (set_alarm_s);
set_alarm_s:
  switch (button)
    {
    case 1:
      goto pick_alarm_i;
    case 2:
      // field_up
      delta = 1;
      goto field_change;
    case 3:
      // field_down
      delta = -1;
    field_change:
      switch (alarm_set_field)
	{
	case ASF_hour:
	  i = unbcd (alarm.hour) + delta;
	  if (i < 0)
	    i = 23;
	  if (i > 23)
	    i = 0;
	  alarm.hour = bcd (i);
	  break;
	case ASF_minute:
	  i = unbcd (alarm.minute) + delta;
	  if (i < 0)
	    i = 59;
	  if (i > 59)
	    i = 0;
	  alarm.minute = bcd (i);
	  break;
	case ASF_sun:
	case ASF_mon:
	case ASF_tue:
	case ASF_wed:
	case ASF_thu:
	case ASF_fri:
	case ASF_sat:
	  alarm.dow ^= (0x40 >> (alarm_set_field - ASF_sun));
	  break;
	case ASF_month:
	  i = unbcd (alarm.month) + delta;
	  if (i < 0)
	    i = 12;
	  if (i > 12)
	    i = 0;
	  alarm.month = bcd (i);
	  if (alarm.day < 1 || alarm.day > 0x31)
	    alarm.day = 1;
	  break;
	case ASF_mday:
	  i = unbcd (alarm.day) + delta;
	  if (i < 1)
	    i = days_per_month[alarm.month];
	  if (i > days_per_month[alarm.month])
	    i = 1;
	  alarm.day = bcd (i);
	  break;
	case ASF_zen:
	  i = alarm.zen + delta;
	  if (i < 0)
	    i = 255;
	  if (i > 255)
	    i = 0;
	  alarm.zen = i;
	  break;
	case ASF_webdest:
	  {
	    word web_addr, val;
	    unsigned int web_size;
	    web_addr = get_toc (TOC_web, &web_size);
	    if (alarm.web == 0)
	      alarm.web = web_addr + web_size;
	    do
	      {
		alarm.web += delta * sizeof (WebDestination);
		if (alarm.web > web_addr + web_size)
		  alarm.web = web_addr;
		if (alarm.web < web_addr)
		  alarm.web = web_addr + web_size;
		if (alarm.web == web_addr + web_size)
		  val = 1;
		else
		  {
		    WebDestination wd;
		    eeprom_read (alarm.web, &wd, sizeof (wd));
		    val = wd.mp3_url_addr | wd.image_url_addr | wd.name_addr;
		  }
	      }
	    while (val == 0);
	    if (alarm.web == web_addr + web_size)
	      alarm.web = 0;
	  }
	  break;
	}
      addr = get_toc (TOC_alarm, 0);
      eeprom_write (addr + alarm_idx * sizeof (alarm), &alarm,
		    sizeof (alarm));
      cprintf ("alarm %d %w %b:%b %b %b/%b %w\n", alarm_idx,
	       addr + alarm_idx * sizeof (alarm), alarm.hour, alarm.minute,
	       alarm.dow, alarm.month, alarm.day, alarm.web);
      goto set_alarm_i2;
    case 4:
    case 0:
    default:
      RESET ();
    case 5:
      // next_field
      asf_clear (alarm_set_field);
      if (alarm_set_field == ASF_webdest)
	alarm_set_field = ASF_hour;
      else
	alarm_set_field++;
      asf_put (alarm_set_field);
      return;
    case 6:
      // prev_field
      asf_clear (alarm_set_field);
      if (alarm_set_field == ASF_hour)
	alarm_set_field = ASF_webdest;
      else
	alarm_set_field--;
      asf_put (alarm_set_field);
      return;
    }

  cprintf ("exit?\n");
  GOTO (idle_s);

 debug_time_i:
  cls ();
  rtc_disp (RTC_HALF, 40, 0);
  local_console = 1;
  gotoxy (4, 5);
  rtc_print_date ();
#ifndef SIM
  adc_debug = 1;
  gotoxy (4, 6);
  cprintf("%P", &AppConfig.MyIPAddr);
#endif
  gotoxy (4, 7);

  addr = get_toc (TOC_config, 0);
  timezone = eeprom_readc (addr + ConfigOffset (tz_offset));
  cprintf("TZ: %d", (int)timezone);

  local_console = 0;
  GOTO (debug_time_s);
 debug_time_s:
  switch (button)
    {
    default:
#ifndef SIM
      adc_debug = 0;
#endif
      RESET ();
    }
}
