/* Copyright 2007, 2008 DJ Delorie <dj@delorie.com>
   Distributed under the terms of the GNU General Public License
   version 3, see COPYING-3.0 for details.
*/
#include "pins.h"
#include "funcs.h"

#define scl	_RB6
#define scl_low()  _TRISB6 = 0
#define scl_high() _TRISB6 = 1

#define sda	_RB5
#define sda_low()  _TRISB5 = 0
#define sda_high() _TRISB5 = 1
#define sda_set(b)  _TRISB5 = (b&1)

static void i2c_stop ();

static int otic = 0;
static char oaddr = 0;
static char active = 0;

// The rule is, all operations tic BEFORE they change.

static void
tic (int delay)
{
  register int t;
  do
    {
      t = TMR2;
    }
  while (t - otic < delay);
  otic = t;
}

#define TIC() tic(19)

static void
i2c_start ()
{
  if (active)
    i2c_stop ();
  //tic(0);
  TIC ();
  sda_low ();
  TIC ();
  scl_low ();
  active = 1;
}

static void
i2c_stop ()
{
  if (!active)
    return;
  sda_low ();
  TIC ();
  scl_high ();
  TIC ();
  sda_high ();
  active = 0;
}

static int
i2c_send (char b)
{
  int i;
  for (i = 7; i >= 0; i--)
    {
      sda_set (b >> i);
      TIC ();
      scl_high ();
      TIC ();
      scl_low ();
    }
  sda_high ();
  TIC ();
  scl_high ();
  while (!scl);
  TIC ();
  i = sda;
  scl_low ();
  return !i;
}

int
i2c_recv (int ack)
{
  int i;
  int b = 0;
  for (i = 7; i >= 0; i--)
    {
      TIC ();
      scl_high ();
      TIC ();
      b <<= 1;
      b |= sda;
      scl_low ();
    }
  sda_set (!ack);
  TIC ();
  scl_high ();
  TIC ();
  scl_low ();
  sda_high ();
  return b;
}

int
i2c_wr (int addr, void *vbuf, uint len)
{
  unsigned char *buf = (unsigned char *) vbuf;
  i2c_stop ();
  oaddr = 0;
  i2c_start ();
  if (!i2c_send (addr * 2))
    goto error;
  while (len > 0)
    {
      if (!i2c_send (*buf))
	break;
      buf++;
      len--;
    }
  i2c_stop ();
  return 0;
error:
  i2c_stop ();
  return 1;
}

int
i2c_wra (int addr, uint ptr, void *vbuf, uint len)
{

  unsigned char *buf = (unsigned char *) vbuf;
  i2c_stop ();
  oaddr = 0;
  i2c_start ();
  //cprintf("i2c addr %b ptr %w\n", addr, ptr);
  if (!i2c_send (addr * 2))
    goto error;
  if (!i2c_send (ptr >> 8))
    goto error;
  if (!i2c_send (ptr & 0xff))
    goto error;
  while (len > 0)
    {
      if (!i2c_send (*buf))
	break;
      buf++;
      len--;
    }
  i2c_stop ();
  return 0;
error:
  i2c_stop ();
  return 1;
}

int
i2c_rd (int addr, void *vbuf, uint len)
{
  unsigned char *buf = (unsigned char *) vbuf;
  i2c_stop ();
  oaddr = 0;
  //cprintf("i2c read %w %d:", addr, len);
  i2c_start ();
  if (!i2c_send (addr * 2 + 1))
    goto error;
  while (len > 0)
    {
      len--;
      *buf++ = i2c_recv (len);
      //cprintf(" %b", buf[-1]);
    }
  //cprintf("\n");
  i2c_stop ();
  return 0;
error:
  i2c_stop ();
  return 1;
}

int
i2c_srd (int addr)
{
  if (oaddr != addr || !active)
    {
      if (active)
	i2c_stop ();
      i2c_start ();
      i2c_send (addr * 2 + 1);
      oaddr = addr;
    }
  return i2c_recv (1);
}

void
i2c_end ()
{
  i2c_recv (0);
  i2c_stop ();
  oaddr = 0;
}
