/* Copyright 2007, 2008 DJ Delorie <dj@delorie.com>
   Distributed under the terms of the GNU General Public License
   version 3, see COPYING-3.0 for details.
*/
#include <string.h>

#include "pins.h"
#include "funcs.h"
#include "globals.h"
#ifndef SIM
#include "TCPIP Stack/TCPIP.h"
#endif /* SIM */

ToDo todo;
Status status;

static int old_volume = -1;
static void volume_task (void);

void
waitus (int i)
{
#ifdef SIM
  sim_usleep (i);
#else /* not SIM */
  while (--i)
    {
      Nop ();
      Nop ();
      Nop ();
      Nop ();
      Nop ();
      Nop ();
      Nop ();
      Nop ();
      Nop ();
      Nop ();
      Nop ();
      Nop ();
    }
#endif /* not SIM */
}

void
waitms (int i)
{
#ifdef SIM
  sim_usleep (i * 1000);
#else /* not SIM */
  while (i--)
    waitus (1000);
#endif /* not SIM */
}

void
io_reset ()
{
  oled_cs = 1;
  reset = 0;
  v12en = 0;

  waitus (30);
  reset = 1;

  waitus (30);
  v12en = 1;

  waitms (30);
}

void
console_send (char ch)
{
#ifdef SIM
  putchar (ch);
#else /* not SIM */
  while (U1STAbits.UTXBF);
  U1TXREG = ch;
#endif /* not SIM */
}

static int need_timestamp = 1;
void
console_send_cr (char ch)
{
  if (ch == '\n')
    {
      console_send ('\r');
      console_send (ch);
      need_timestamp = 1;
    }
  else
    {
      if (need_timestamp)
	{
	  console_hex (hour12);
	  console_send (':');
	  console_hex (minute);
	  console_send (':');
	  console_hex (second);;
	  console_send (' ');
	  need_timestamp = 0;
	}
      console_send (ch);
    }
}

void
console_wait ()
{
#ifndef SIM
  while (U1STAbits.UTXBF);
#endif /* not SIM */
}

void
console_nibble (int i)
{
  static const char hex[] = "0123456789abcdef";
  console_send (hex[i & 15]);
}

void
console_hex (int i)
{
  //console_send(i>>8);
  //console_send(i);
  //console_nibble (i>>12);
  //console_nibble (i>>8);
  console_nibble (i >> 4);
  console_nibble (i);
}

#ifdef SIM
int
main (int argc, char **argv)
{
  sim_init (argc, argv);
  main2 ();
}

#define main main2
#endif /* SIM */

/* CODE SAMPLE: main.exc */

int
main ()
{
  int myDHCPBindCount = 0;

  configure_pic_io ();
  io_reset ();

  console_send(0x55);
  console_send(0xAA);

  configure_oled ();

  oled_image (get_toc (TOC_image_intro, 0), 0, 0);
  cprintf("\nDJ's Alarm Clock\n");
  waitms (1000);

  TickInit ();
  InitAppConfig ();
  StackInit ();

  // Configure LEDA to display TX activity, LEDB to display RX activity
  SetLEDConfig (0x3120);

  eeprom_init ();
  mp3_init ();
  rtc_init ();

  memset (&todo, 0, sizeof (todo));

  rtc_disp (RTC_MAIN, 64, 0);

  button_autorepeat = 0x1c;

  while (1)
    {
#ifndef SIM
      StackTask ();

      if (DHCPBindCount != myDHCPBindCount)
	{
	  myDHCPBindCount = DHCPBindCount;
	  todo.check_net_time = 1;
	  todo.new_net_time = 1;
	  // cprintf("IP: %P\n", &AppConfig.MyIPAddr);
	}
#endif /* SIM */

      rtc_task ();
      adc_task ();
      mp3_task ();
      eeprom_task ();
      alarm_task ();

      if (todo.check_volume)
	volume_task ();

      if (button_event)
	{
	  button_event = 0;
	  cprintf("button %d\n", button);
	  if (button == 7)
	    alarm_snooze ();
	  else
	    gui (button & 7);
	}
    }
}

/* END CODE SAMPLE */

static void
volume_task (void)
{
  static int vol_timeout = 0;

  if (status.muted)
    {
      todo.reset_volume_val = 1;
      if (todo.reset_volume_bar)
	{
	  todo.reset_volume_bar = 0;
	  if (rtc_current_mode () == RTC_VOL)
	    rtc_disp (RTC_MAIN, 64, 0);
	}
      return;
    }

  if (old_volume == -1 || todo.reset_volume_val)
    {
      old_volume = volume;
      todo.reset_volume_val = 0;
      return;
    }
  switch (rtc_current_mode ())
    {
    case RTC_MAIN:
      if (old_volume > volume - 20 && old_volume < volume + 20)
	break;
      rtc_disp (RTC_VOL, 0, 0);
      cls ();
      oled_image (get_toc (TOC_image_svolume, 0), 0, 0);
      todo.reset_volume_bar = 1;

      /* Fall through.  */
    case RTC_VOL:
      if (old_volume != volume)
	{
	  uint bar = ((uint) volume * 62U + (volume >> 1)) >> 9;
	  uint bar2 = bar / 2;

	  if (bar2 > 0)
	    oled_fill (2, 40, bar2 * 2 + 1, 56, 0xff);
	  if (bar & 1)
	    {
	      oled_fill (bar2 * 2 + 2, 40, bar2 * 2 + 3, 56, 0xf3);
	      if (bar2 < 61)
		oled_fill (bar2 * 2 + 4, 40, 125, 56, 0x33);
	    }
	  else if (bar2 < 62)
	    oled_fill (bar2 * 2 + 2, 40, 125, 56, 0x33);
	  old_volume = volume;
	  vol_timeout = eseconds + 2;
	}
      if (vol_timeout - eseconds < 0)
	{
	  rtc_disp (RTC_MAIN, 64, 0);
	}
      break;

    default:
      old_volume = volume;
      break;
    }
}
