/* Copyright 2007, 2008 DJ Delorie <dj@delorie.com>
   Distributed under the terms of the GNU General Public License
   version 3, see COPYING-3.0 for details.
*/
#include <string.h>

#include "pins.h"
#include "funcs.h"
#include "globals.h"

#if 0
// Atlanta settings
static const unsigned char oled_init[] = {
  0x15, 0x00, 0x3f,		// col address
  0x75, 0x00, 0x3f,		// row address
  0x81, 0x33,			// contrast control (max 0x33)
  0x86,				// current range = quarter=84,half=85,full=86
  0xa0, 0x52,			// remap
  0xa1, 0x00,			// start line
  0xa2, 0x40,			// display offset
  0xa8, 0x3f,			// multiplex ratio
  0xb2, 0x46,			// row period
  0xaf,				// display on
  0xa4,				// display mode: normal
  0xad, 0x02,			// master configuration, dc-dc
  0xcf, 0xf0,			// dc-dc bias
  0xb0, 0x08,			// precharge compensation enable
  0xb4, 0x00,			// precharge compensation level
  0xb3, 0x41,			// clock divide
  0xb1, 0x22,			// phase length
  0xbf, 0xed,			// vsl
  0xbe, 0x00,			// VcomH
  0xbc, 0x0b,			// Vprecharge
#if 0
  0xb8,				// grey scale table
  0x01, 0x01, 0x01, 0x02,
  0x20, 0x02, 0x30, 0x03,
  0x40, 0x04, 0x50, 0x05,
  0x60, 0x06, 0x70
#else
  0xb8, 0x11, 0x11, 0x11,	// grey scale table
  0x11, 0x22, 0x22, 0x33,
  0x33, 0x44, 0x44, 0x55,
  0x55, 0x66, 0x55, 0x77
#endif
};
#else
// Calgary settings
static const unsigned char oled_init[] = {
  0x15, 0x00, 0x3f,		// col address
  0x75, 0x00, 0x3f,		// row address
  0x81, 0x66,			// contrast control
  0x86,				// current range = quarter=84,half=85,full=86
  0xa0, 0x41,			// remap
  0xa1, 0x00,			// start line
  0xa2, 0x44,			// display offset
  0xa4,				// display mode: normal
  //0xa7,                       // display mode: inverted
  0xa8, 0x3f,			// multiplex ratio
  0xb1, 0x22,			// phase length
  0xb2, 0x46,			// row period
  0xb3, 0xf1,			// clock divide
  0xbf, 0x0d,			// vsl
  0xbe, 0x02,			// VcomH
  0xbc, 0x10,			// Vprecharge
  0xad, 0x02,			// master configuration, dc-dc
  0xcf, 0xf0,			// dc-dc bias
  0xb0, 0x28,			// precharge compensation enable
  0xb4, 0x07,			// precharge compensation level
  0xaf,				// display on
#if 0
  0xb8,				// grey scale table
  0x01, 0x01, 0x01, 0x02,
  0x20, 0x02, 0x30, 0x03,
  0x40, 0x04, 0x50, 0x05,
  0x60, 0x06, 0x70
#else
  0xb8, 0x00, 0x11, 0x11,	// grey scale table
  0x11, 0x22, 0x22, 0x33,
  0x33, 0x44, 0x44, 0x55,
  0x55, 0x66, 0x55, 0x88
#endif
};
#endif

void
configure_oled ()
{
#ifndef SIM
  spi_oled_command ();
  ospi_putc (0xae);
  ospi_puts ((const char *) oled_init, sizeof (oled_init));
  spi_done ();
#endif /* SIM */
  cls ();
}

static const unsigned char oled_origin[] = {
  0x15, 0x00, 0x3f,		// col address
  0x75, 0x00, 0x3f,		// row address
};

// [row][col]
static unsigned char sram[8][21];
static unsigned char srow = 0, scol = 0;
static char bold = 0x80;

static void
oled_window (int left, int right, int top, int bottom)
{
  spi_oled_command ();
  ospi_putc (0x15);
  ospi_putc (left);
  ospi_putc (right);
  ospi_putc (0x75);
  ospi_putc (top);
  ospi_putc (bottom);
  spi_oled_data ();
}

void
cls ()
{
  int i;
  oled_window (0, 63, 0, 63);
  for (i = 0; i < 64 * 64; i++)
    ospi_putc (0x00);
  spi_done ();
  memset (sram, ' ', sizeof (sram));
  srow = scol = 0;
  bold = 0;
}

void
put_c_xy (int c, int x, int y)
{
  static unsigned char _pix[8] =
    { 0x00, 0x70, 0x07, 0x77, 0x00, 0xf0, 0x0f, 0xff };
  byte font[6];
  uint addr;
  unsigned char *pix;
  unsigned char *f;
  int i, b;

  pix = (c & 0x80) ? _pix + 4 : _pix;
  c &= 0x7f;

  addr = get_toc (TOC_console_font, 0);
  eeprom_read (addr + c * 6, font, 6);

  x *= 3;
  y *= 8;

  oled_window (x, x + 2, y, y + 7);

  f = font;
  for (i = 0; i < 6; i++)
    {
      b = *f++;
      ospi_putc (pix[(b) & 3]);
      ospi_putc (pix[(b >> 2) & 3]);
      ospi_putc (pix[(b >> 4) & 3]);
      ospi_putc (pix[(b >> 6) & 3]);
    }
  spi_done ();
}

static void
scroll ()
{
  int x, y;
  memcpy (sram, &sram[1], 7 * 21);
  memset (sram[7], ' ', 21);
  for (y = 0; y < 8; y++)
    for (x = 0; x < 21; x++)
      put_c_xy (sram[y][x], x, y);
  srow--;
}

void
cput_c (char c)
{
  if (c == 014)			// CTRL-L
    {
      bold = 0;
      cls ();
    }
  else if (c == '\1')
    bold = 0x80;
  else if (c == '\2')
    bold = 0x00;
  else if (c == '\n')
    {
      scol = 0;
      srow++;
      if (srow > 8)
	scroll ();
    }
  else if (c == '\r')
    {
      scol = 0;
    }
  else if (c >= 0)
    {
      c |= bold;
      if (scol >= 21)
	cput_c ('\n');
      while (srow > 7)
	scroll ();
      sram[srow][scol] = c;
      put_c_xy (c, scol, srow);
      scol++;
    }
}

void
cput_cs (const char *s)
{
  while (*s)
    cput_c (*s++);
}

void
gotoxy (int x, int y)
{
  if (x >= 0 && x <= 20 && y >= 0 && y <= 7)
    {
      scol = x;
      srow = y;
    }
}

void
oled_fill (int x1, int y1, int x2, int y2, int val)
{
  int i;
  x1 /= 2;
  x2 /= 2;
  oled_window (x1, x2, y1, y2);
  for (i = (x2 - x1 + 1) * (y2 - y1 + 1); i > 0; i--)
    ospi_putc (val);
  spi_done ();
}

static byte top_shadows[16] =
  { 1, 4, 5, 7, 8, 9, 10, 11, 12, 12, 13, 14, 14, 15, 11, 12 };
static byte bot_shadows[16] =
  { 1, 2, 3, 1, 2, 2, 3, 3, 4, 5, 6, 6, 7, 8, 8, 8 };
static byte oled_4bit_pixels[4] = { 0, 7, 15, 11 };

static byte oled_1bit_pixels[4] = { 0x00, 0xf0, 0x0f, 0xff };

#define GET() i2c_srd(EEPROM_I2C)

void
oled_image (uint iaddr, int x, int y)
{
  ImageHeader header;
  byte b, i, p;

  eeprom_read (iaddr, &header, 3);

  //cprintf("img %w %d,%d : %c %dx%d: ", iaddr, x, y, header.type, header.width, header.height);
  oled_window (x / 2, (x + header.width) / 2 - 1, y, y + header.height - 1);

  spi_oled_data ();

  switch (header.type)
    {
    case 'A':
      while (1)
	{
	  b = GET ();
	  if (b & 0x80)
	    {
	      char p[4];
	      i = b & 0x7f;
	      b = GET ();
	      p[0] = oled_1bit_pixels[b & 3];
	      p[1] = oled_1bit_pixels[(b >> 2) & 3];
	      p[2] = oled_1bit_pixels[(b >> 4) & 3];
	      p[3] = oled_1bit_pixels[(b >> 6) & 3];
	      while (i--)
		ospi_puts (p, 4);
	    }
	  else if (b)
	    {
	      i = b;
	      while (i--)
		{
		  b = GET ();
		  ospi_putc (oled_1bit_pixels[b & 3]);
		  ospi_putc (oled_1bit_pixels[(b >> 2) & 3]);
		  ospi_putc (oled_1bit_pixels[(b >> 4) & 3]);
		  ospi_putc (oled_1bit_pixels[(b >> 6) & 3]);
		}
	    }
	  else
	    break;
	}
      break;
    case 'B':
      while (1)
	{
	  b = GET ();
	  if (b & 0xf0)
	    {
	      i = (b >> 4);
	      b &= 0x0f;
	      p = oled_4bit_pixels[b >> 2] | (oled_4bit_pixels[b & 3] << 4);
	      while (i--)
		ospi_putc (p);
	    }
	  else if (b)
	    {
	      i = b;
	      while (i--)
		{
		  b = GET ();
		  p =
		    oled_4bit_pixels[(b >> 4) & 3] |
		    (oled_4bit_pixels[(b >> 6) & 3] << 4);
		  ospi_putc (p);
		  p =
		    oled_4bit_pixels[(b >> 0) & 3] |
		    (oled_4bit_pixels[(b >> 2) & 3] << 4);
		  ospi_putc (p);
		}
	    }
	  else
	    break;
	}
      break;
    case 'C':
      while (1)
	{
	  b = GET ();
	  if (b & 0xf0)
	    {
	      i = (b >> 4);
	      b &= 0x0f;
	      p = b | (b << 4);
	      while (i--)
		ospi_putc (p);
	    }
	  else if (b)
	    {
	      i = b;
	      while (i--)
		ospi_putc (GET ());
	    }
	  else
	    break;
	}
    default:
      mp3_cs = 1;
      mp3_cs = 0;
      break;
    }
  spi_done ();
  i2c_end ();
}

void
oled_set_intensity ()
{
  oled_4bit_pixels[1] = bot_shadows[photocell];
  oled_4bit_pixels[2] = top_shadows[photocell];
  oled_4bit_pixels[3] = photocell;

  spi_oled_command ();
  ospi_putc (photocell ? 0x86 : 0x84);
  spi_done ();
}
