/* Copyright 2007, 2008 DJ Delorie <dj@delorie.com>
   Distributed under the terms of the GNU General Public License
   version 3, see COPYING-3.0 for details.
*/
#include "pins.h"
#include "funcs.h"
#include "globals.h"
#define __TCP_C 1
#ifndef SIM
#include "TCPIP Stack/TCPIP.h"
#endif /* not SIM */

byte rtc_tick = 0;
int eseconds = 0;
int sleep_time = 0;

#define NUM_GLYPHS 15

static RTC_DISP disp_mode;
static byte disp_x, disp_y, disp_font;
static word font[NUM_GLYPHS];
static byte widths[NUM_GLYPHS];
static byte leftmost, rightmost, height;
static byte thm_12hour, thour, tminute, display_is_static = 0;

static char *down[] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};

#ifndef SIM
static UDP_SOCKET TIMESocket = INVALID_UDP_SOCKET;
#endif /* not SIM */

static byte bcd2ampm[] = {
  0x12, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09,
  0, 0, 0, 0, 0, 0,
  0x10, 0x11, 0x12, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
  0, 0, 0, 0, 0, 0,
  0x08, 0x09, 0x10, 0x11
};

byte hour, hour12 = 0x12, pm, minute, second;
byte month, day, year, dow;

// 00:00:00 Jan 1 2000 GMT
#define IOFFSET 3155745600UL

int
hour_24_to_12 (int hour, byte * pm)
{
  *pm = (hour > 0x11) ? 1 : 0;
  return bcd2ampm[hour];
}

#ifndef SIM
static void
rtc_write (int which, unsigned int i)
{
  RCFGCALbits.RTCPTR = which;
  RTCVAL = i;
}

static void
rtc_write2 (int which, unsigned int msb, unsigned int lsb)
{
  RCFGCALbits.RTCPTR = which;
  RTCVAL = (msb << 8) | lsb;
}

static void
rtc_read2 (int which, unsigned char *ptr)
{
  unsigned int i;
  RCFGCALbits.RTCPTR = which;
  i = RTCVAL;
  ptr[1] = i >> 8;
  ptr[0] = i;
}

void
rtc_init ()
{
  word addr;

  __builtin_write_OSCCONL (0x02);
  NVMKEY = 0x55;
  NVMKEY = 0xaa;
  RCFGCALbits.RTCWREN = 1;
  RCFGCALbits.RTCEN = 1;
  addr = get_toc (TOC_config, 0);
  _CAL = eeprom_readc (addr + ConfigOffset (rtc_calib));

  rtc_write (0, 0);
  rtc_write (1, 0);
  rtc_write (2, 0);
  rtc_write (3, 0);

  ALCFGRPTbits.ALRMEN = 1;
  ALCFGRPTbits.CHIME = 1;
  ALCFGRPTbits.AMASK = 1;

  IFS3bits.RTCIF = 0;
  IEC3bits.RTCIE = 1;
  IPC15bits.RTCIP = 5;
}
#endif /* not SIM */

static int
max_widths (int from, int to)
{
  int i, rv = 0;
  for (i = from; i <= to; i++)
    rv = MAX (rv, widths[i]);
  return rv;
}

static void
rtc_disp_int (RTC_DISP disp, int x, int y)
{
  int newfont, total_width, i, ix, iy;
  byte str[6], s1, disp_24hr;
  word addr;

  addr = get_toc (TOC_config, 0);
  disp_24hr = eeprom_readc (addr + ConfigOffset (disp_24hr));

  disp_mode = disp;
  disp_x = x;
  disp_y = y;

  switch (disp_mode)
    {
    case RTC_MAIN:
      newfont = photocell ? TOC_main_font : TOC_night_font;
      break;
    case RTC_HALF:
      newfont = TOC_half_font;
      break;
    case RTC_MINI:
      newfont = TOC_mini_font;
      break;
    default:
      return;
    }

  if (newfont != disp_font)
    {
      int i;
      int max_width;
      unsigned int font_addr;
      disp_font = newfont;
      font_addr = get_toc (newfont, 0);
      eeprom_read (font_addr, font, sizeof (font));
      for (i = 0; i < NUM_GLYPHS; i++)
	eeprom_read (font[i] + 1, widths + i, 1);
      eeprom_read (font[0] + 2, &height, 1);
      max_width = (widths[1]
		   + max_widths (0, 2)
		   + widths[10] + max_widths (0, 5) + max_widths (0, 9));
      leftmost = disp_x - max_width / 2;
      rightmost = leftmost + max_width - 1;
      if (newfont == TOC_main_font || newfont == TOC_night_font)
	{
	  cls();
	  leftmost = 0;
	  rightmost = 127;
	}
    }

  if (display_is_static)
    {
      byte pm, h12;

      if (thm_12hour)
	{
	  h12 = hour_24_to_12 (thour, &pm);
	  str[0] = h12 >> 4;
	  s1 = str[0] ? 0 : 1;
	  str[1] = h12 & 0x0f;
	  str[2] = pm ? PM : AM;
	  if (tminute == 0 && h12 == 0x12)
	    {
	      if (pm)
		str[2] = NOON;
	      else
		str[2] = MID;
	    }
	}
      else
	{
	  str[0] = thour >> 4;
	  s1 = str[0] ? 0 : 1;
	  str[1] = thour & 0x0f;
	  str[2] = 12;
	}
      str[3] = tminute >> 4;
      str[4] = tminute & 0x0f;
      str[5] = 0;
    }
  else if (disp_24hr)
    {
      str[0] = hour >> 4;
      s1 = str[0] ? 0 : 1;
      str[1] = hour & 0x0f;
      str[2] = 12;
      str[3] = minute >> 4;
      str[4] = minute & 0x0f;
    }
  else
    {
      str[0] = hour12 >> 4;
      s1 = str[0] ? 0 : 1;
      str[1] = hour12 & 0x0f;
      str[2] = pm ? PM : AM;
      str[3] = minute >> 4;
      str[4] = minute & 0x0f;

      if (minute == 0 && hour12 == 0x12)
	{
	  if (pm)
	    str[2] = NOON;
	  else
	    str[2] = MID;
	}
    }

  total_width = 0;
  for (i = s1; i < 5; i++)
    total_width += widths[str[i]];
  ix = disp_x - (total_width / 2 & ~1);
  iy = disp_y;

  /* The next couple of operations take a while, so make sure we keep
     the mp3 buffer full (if we can).  We're using the i2c bus and
     SPI1, the mp3 uses SPI2.  */
  if (ix > leftmost)
    {
      mp3_process_buffer ();
      oled_fill (leftmost, iy, ix - 1, iy + height - 1, 0);
    }
  for (i = s1; i < 5; i++)
    {
      int ox = ix;

      /* This code just ensures that we don't try to draw outside our
	 window.  I.e. an ugly display is better than a corrupt
	 display.  */
      if (ix < 0)
	ix = 0;
      else if (ix+widths[str[i]] > 128)
	ix = 128 - widths[str[i]];

      mp3_process_buffer ();
      oled_image (font[str[i]], ix, iy);
      ix += widths[str[i]];
    }
  if (ix < rightmost)
    {
      mp3_process_buffer ();
      oled_fill (ix, iy, rightmost, iy + height - 1, 0);
    }
  mp3_process_buffer ();
}

RTC_DISP
rtc_current_mode (void)
{
  return disp_mode;
}

void
rtc_disp (RTC_DISP disp, int x, int y)
{
  display_is_static = 0;
  rtc_disp_int (disp, x, y);
}

void
rtc_redisp ()
{
  if (!display_is_static)
    rtc_disp_int (disp_mode, disp_x, disp_y);
}

void
rtc_disp_hm (RTC_DISP disp, int x, int y, byte hour, byte minute)
{
  display_is_static = 1;
  thour = hour;
  tminute = minute;
  thm_12hour = 1;
  rtc_disp_int (disp, x, y);
}

void
rtc_disp_hm24 (RTC_DISP disp, int x, int y, byte hour, byte minute)
{
  display_is_static = 1;
  thour = hour;
  tminute = minute;
  thm_12hour = 0;
  rtc_disp_int (disp, x, y);
}

void
rtc_pause (int paused)
{
  switch (paused)
    {
    case 1:
      disp_mode |= 0x80;
      break;
    case 0:
      disp_mode &= 0x7f;
      break;
    case -1:
      break;
    }
  /* Force a re-read of the font, in case we're "pausing" for an
     eeprom update.  */
  disp_font = 0;
}

static byte allow_updates = 0;

void __attribute__ ((interrupt)) _RTCCInterrupt (void)
{
#ifndef SIM
  IFS3bits.RTCIF = 0;
#if 1
  _LATB10 = 1;
  _LATB10 = 0;
#endif
#endif /* not SIM */
  rtc_tick = 1;
  eseconds++;
  // The last 1->0 transition happens outside of the interrupt handler.
  if (sleep_time > 1)
    sleep_time--;
}

#ifndef SIM
// These two typedefs are for reference only

typedef struct
{
  unsigned long sec;
  unsigned long frac;
} Time;

typedef struct
{
  byte li_vn_mode;
  byte stratum;
  byte poll;
  char precision;
  long root_delay;
  long root_dispersion;
  long resource_identifier;
  Time reference_timestamp;
  Time originate_timestamp;
  Time receive_timestamp;
  Time transmit_timestamp;
} SNTP;

// This is the timestamp we use as the "send" timestamp, so we can
// tell (1) that the response is legitimate, and (2) so we can ignore
// extra responses once we have a good one.  We reset it when we're
// not expecting more responses.
static long last_ntp_sent = 0;

static void
UDPIgnore (int n)
{
  byte b;
  while (n--)
    UDPGet (&b);
}

static void
ntp_send ()
{
  int i;
  if (UDPIsPutReady (TIMESocket) >= 48)
    {
      UDPPut ((3 << 0) | (4 << 3));	// li, vn 4, mode client
      for (i = 0; i < 3 + 3 * 4 + 3 * 8; i++)
	UDPPut (0);		// stratum through receive_timestamp
      last_ntp_sent = eseconds;
      //cprintf("%d s %x\n", eseconds, &last_ntp_sent, 4);
      UDPPutArray ((BYTE *) & last_ntp_sent, 4);	// send current time to match later
      UDPPut (0);
      UDPPut (0);
      UDPPut (0);
      UDPPut (0);
      UDPFlush ();

      // If this is just an update, only send one request.
      if (!todo.new_net_time)
	todo.check_net_time = 0;
    }
}

static void
ntp_recv ()
{
  if (TIMESocket == INVALID_UDP_SOCKET)
    return;
  if (UDPIsGetReady (TIMESocket) >= 48 && last_ntp_sent)
    {
      byte server_version;
      unsigned long match_ts;
      byte sent_time[4];
      byte isec, imin, ihour;
      byte iday, imon, iyear;
      byte idow;
      unsigned long iseconds;
      char timezone;
      word addr;

      UDPGet (&server_version);
      UDPIgnore (3 + 3 * 4 + 1 * 8);
      UDPGetArray ((BYTE *) & match_ts, 4);
      UDPIgnore (4 + 8);
      UDPGetArray ((BYTE *) & sent_time, 4);
      UDPDiscard ();
      //cprintf("%d r %b %x\n", eseconds, server_version, &match_ts, 4);

      if ((server_version & 0x3f) != 0x24)
	return;
      if (match_ts != last_ntp_sent)
	return;

      // ignore any further responses from that query
      last_ntp_sent = 0;

      todo.check_net_time = 0;
      todo.new_net_time = 0;

      iseconds = (((unsigned long) sent_time[0] << 24ul)
		  | ((unsigned long) sent_time[1] << 16ul)
		  | ((unsigned long) sent_time[2] << 8ul)
		  | ((unsigned long) sent_time[3]));
      //cprintf("isec %x\n", &iseconds, 4);
      iseconds -= IOFFSET;
      addr = get_toc (TOC_config, 0);
      timezone = eeprom_readc (addr + ConfigOffset (tz_offset));
      iseconds += timezone * 60*60;

      isec = iseconds % 60;
      iseconds /= 60;

      imin = iseconds % 60;
      iseconds /= 60;

      iseconds -= 4;

      ihour = iseconds % 24;
      iseconds /= 24;

      idow = iseconds % 7;

      iyear = iseconds * 4 / 1461;	/* 365.25*4 */
      iseconds -= (iyear * 365ul + ((iyear-1) / 4));

      imon = 1;

#define DOM(n) if (iseconds >= (n)) { iseconds -= (n); imon ++; }

      DOM (31);			/* Jan */
      DOM (iyear & 3 ? 28 : 29);	/* Feb */
      DOM (31);			/* Mar */
      DOM (30);			/* Apr */
      DOM (31);			/* May */
      DOM (30);			/* Jun */
      DOM (31);			/* Jul */
      DOM (31);			/* Aug */
      DOM (30);			/* Sep */
      DOM (31);			/* Oct */
      DOM (30);			/* Nov */
      DOM (31);			/* Dec */
      iday = iseconds + 1;

#if 0
      {
	cprintf ("NNTP: %d-%d-%d %s %d:%d:%d\n",
		 iyear+2000, imon, iday, down[idow], ihour, imin, isec);
      }
#endif

      rtc_write2 (0, bcd (imin), bcd (isec));
      rtc_write2 (1, bcd (idow), bcd (ihour));
      rtc_write2 (2, bcd (imon), bcd (iday));
      rtc_write2 (3, 0, bcd (iyear));
      return;
    }
  else if (UDPIsGetReady (TIMESocket) > 0)
    UDPDiscard ();
}
#endif /* not SIM */

void
rtc_task ()
{
  int minute_tick = 0;
  byte t[2];

  ntp_recv ();

  if (!(rtc_tick || todo.rtc_need_redisplay))
    return;
  rtc_tick = 0;

  if (sleep_time == 1)
    sleep_off ();
  update_sleep_time_display ();

  rtc_read2 (0, t);
  if (minute != t[1])
    minute_tick = 1;
  minute = t[1];
  second = t[0];

  rtc_read2 (1, t);
  dow = t[1];
  hour = t[0];
  hour12 = bcd2ampm[hour];
  pm = (hour > 0x11) ? 1 : 0;

  rtc_read2 (2, t);
  month = t[1];
  day = t[0];

  rtc_read2 (3, t);
  year = t[0];

#if 0
  put_c_xy ((hour >> 4) + '0', 13, 0);
  put_c_xy ((hour & 15) + '0', 14, 0);
  put_c_xy (':', 15, 0);
  put_c_xy ((minute >> 4) + '0', 16, 0);
  put_c_xy ((minute & 15) + '0', 17, 0);
  put_c_xy (':', 18, 0);
  put_c_xy ((second >> 4) + '0', 19, 0);
  put_c_xy ((second & 15) + '0', 20, 0);
#endif

  /*cprintf("Time: %b:%b:%b 20%b-%b-%b\n", hour, minute, second, year, month, day);*/

  if (second == 0x30 && allow_updates)
    todo.check_net_time = 1;

  if (minute_tick)
    {
      rtc_redisp ();
      alarm_check ();
    }
  else if (todo.rtc_need_redisplay)
    {
      rtc_redisp ();
      todo.rtc_need_redisplay = 0;
    }

#ifndef SIM
  if (todo.check_net_time)
    {
      allow_updates = 1;
      if (TIMESocket == INVALID_UDP_SOCKET)
	TIMESocket = UDPOpen (123, NULL, 123);
      if (TIMESocket == INVALID_UDP_SOCKET)
	return;

      ntp_send ();
    }
#endif /* not SIM */

}

void
rtc_print_date ()
{
  int dow, month, day, year;
  char t[2];

  rtc_read2 (1, t);
  dow = t[1];

  rtc_read2 (2, t);
  month = t[1];
  day = t[0];

  rtc_read2 (3, t);
  year = t[0];

  cprintf("%s 20%b-%b-%b\n", down[dow], year, month, day);
}
