/* Copyright 2007, 2008 DJ Delorie <dj@delorie.com>
   Distributed under the terms of the GNU General Public License
   version 3, see COPYING-3.0 for details.
*/
#include "pins.h"
#include "funcs.h"

static void
spi_wait ()
{
  while (SPI1STATbits.SPITBF == 1)
    ;
}

static void
ospi_wait ()
{
  while (SPI2STATbits.SPITBF == 1)
    ;
}

static void
spi_flush ()
{
  while (SPI1STATbits.SPIBEC || !SPI1STATbits.SRMPT)
    ;
  while (SPI2STATbits.SPIBEC || !SPI2STATbits.SRMPT)
    ;
}

void
spi_oled_command ()
{
  spi_flush ();
  mp3_cs = 0;
  ether_cs = 1;
  oled_dc = 0;
  oled_cs = 0;
}

void
spi_oled_data ()
{
  spi_flush ();
  mp3_cs = 0;
  ether_cs = 1;
  oled_dc = 1;
  oled_cs = 0;
}

void
spi_mp3 ()
{
  spi_flush ();
  mp3_cs = 1;
  ether_cs = 1;
  oled_dc = 1;
  oled_cs = 1;
}

void
spi_net ()
{
  spi_flush ();
  mp3_cs = 0;
  ether_cs = 0;
  oled_dc = 1;
  oled_cs = 1;
}

void
spi_putc (char byte)
{
  spi_wait ();
  SPI1BUF = byte;
}

void
spi_puts (const char *byte, int len)
{
  while (len--)
    {
      spi_wait ();
      SPI1BUF = *byte++;
    }
}

void
ospi_putc (char byte)
{
  ospi_wait ();
  SPI2BUF = byte;
}

void
ospi_puts (const char *byte, int len)
{
  while (len--)
    {
      ospi_wait ();
      SPI2BUF = *byte++;
    }
}

void
spi_done ()
{
  int j;

  spi_flush ();
  mp3_cs = 0;
  ether_cs = 1;
  oled_cs = 1;
  ether_if = 0;

  /* Clear out any received bytes. */
  while (!SPI1STATbits.SRXMPT)
    j = SPI1BUF;
  SPI1STATbits.SPIROV = 0;
  IFS0bits.SPI1IF = 0;

  /* Clear out any received bytes. */
  while (!SPI2STATbits.SRXMPT)
    j = SPI2BUF;
  SPI2STATbits.SPIROV = 0;
  IFS2bits.SPI2IF = 0;
}
